﻿using System;
using System.Windows.Data;
using System.Windows.Markup;

namespace LocalizationDemo.Localization
{
    public class LocExtension : MarkupExtension
    {
        /// <summary>
        /// Gets or sets the key to use to look up the appropriate value to display.
        /// </summary>
        /// <remarks>
        /// Either this or <see cref="KeyBinding"/> must be set.
        /// </remarks>
        public string Key { get; set; }

        /// <summary>
        /// Gets or sets the key binding to use the look up the appropriate value to display.
        /// </summary>
        /// <remarks>
        /// The binding is evaluated, and the result used as the key.
        /// Either this or <see cref="Key"/> must be set.
        /// </remarks>
        public Binding KeyBinding { get; set; }

        /// <summary>
        /// Gets or sets an optional binding to use to supply a single parameter to substitute into the value to display.
        /// </summary>
        /// <remarks>
        /// This or <see cref="ParamBindings"/> may be set, but not both.
        /// </remarks>
        public Binding ParamBinding { get; set; }

        /// <summary>
        /// Gets or set an optional MultiBinding to use to supply multiple parameters to substitute into the value to display.
        /// </summary>
        /// <remarks>
        /// This or <see cref="ParamBinding"/> may be set, but not both.
        /// </remarks>
        public MultiBinding ParamBindings { get; set; }

        public LocExtension()
        {
        }

        public LocExtension(string key)
        {
            this.Key = key;
        }

        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            if (this.Key == null && this.KeyBinding == null)
                throw new ArgumentException("Either Key or KeyBinding must be set");
            if (this.Key != null && this.KeyBinding != null)
                throw new ArgumentException("Either Key or KeyBinding must be set, but not both");
            if (this.ParamBinding != null && this.ParamBindings != null)
                throw new ArgumentException("ParamBinding and ParamBindings may not be set at the same time");

            // Most of these conditions are redundent, according to the assertions above. However I'll still state them,
            // for clarity.

            // A static key, and no values
            if (this.Key != null && this.KeyBinding == null && this.ParamBinding == null && this.ParamBindings == null)
            {
                // Just returning a string!
                return Localizer.Translate(this.Key);
            }
            // A static key, and a single value
            if (this.Key != null && this.KeyBinding == null && this.ParamBinding != null && this.ParamBindings == null)
            {
                var converter = new StaticKeySingleValueConverter() { Key = this.Key, Converter = this.ParamBinding.Converter };
                this.ParamBinding.Converter = converter;
                return this.ParamBinding.ProvideValue(serviceProvider);
            }
            // A static key, and multiple values
            if (this.Key != null && this.KeyBinding == null && this.ParamBinding == null && this.ParamBindings != null)
            {
                var converter = new StaticKeyMultipleValuesConverter() { Key = this.Key, Converter = this.ParamBindings.Converter };
                this.ParamBindings.Converter = converter;
                return this.ParamBindings.ProvideValue(serviceProvider);
            }
            // A bound key, no values
            if (this.Key == null && this.KeyBinding != null && this.ParamBinding == null && this.ParamBindings == null)
            {
                var converter = new BoundKeyNoValuesConverter() { Converter = this.KeyBinding.Converter };
                this.KeyBinding.Converter = converter;
                return this.KeyBinding.ProvideValue(serviceProvider);
            }
            // A bound key, and one value
            if (this.Key == null && this.KeyBinding != null && this.ParamBinding != null && this.ParamBindings == null)
            {
                var converter = new BoundKeyWithValuesConverter();
                var multiBinding = new MultiBinding() { Converter = converter };
                multiBinding.Bindings.Add(this.KeyBinding);
                multiBinding.Bindings.Add(this.ParamBinding);
                return multiBinding.ProvideValue(serviceProvider);
            }
            // A bound key, and multiple values
            if (this.Key == null && this.KeyBinding != null && this.ParamBinding == null && this.ParamBindings != null)
            {
                var converter = new BoundKeyWithValuesConverter() { ValuesConverter = this.ParamBindings.Converter };
                this.ParamBindings.Bindings.Insert(0, this.KeyBinding);
                this.ParamBindings.Converter = converter;
                return this.ParamBindings.ProvideValue(serviceProvider);
            }

            throw new Exception("Should never get here");
        }
    }
}
